# Copyright 2017 by Teem, and other contributors,
# as noted in the individual source code files.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
setup(
        name="gavl",
        version="0.1.2",
        description="Data definition language and aggregation layer",
        packages=find_packages(),
        include_package_data=True,
        install_requires=[
            'pyparsing',
            'click',
            'sqlalchemy',
            'pandas',
            ],
        entry_points='''
            [console_scripts]
            gavl=gavl.cli:main
        ''',
        license='Apache-2.0',
        classifiers=[
            'License :: OSI Approved :: Apache Software License',
        ]
    )
